/***************************************************************************
                          credits.cpp  -  description
                             -------------------
    begin                : Tue Mar 02 2004
    copyright            : (C) 2004 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "credits.h"

#include "defines.h"
#include "cdp.h"
#include "taxi.h"
#include "passenger.h"
#include "level.h"
#include "splashscreen.h"
#include "qcolor.h"

#include <qfont.h>
#include <qcanvas.h>

Credits::Credits(const QString & mission_title, const QString & mission_author, const Taxipilot::Taxis & taxis, const Taxipilot::Passengers & passengers, const QValueList<Taxipilot::LevelHandle> & levels) : TimedObject() {
	unsigned int i;
	QFont credits_font = QFont(CREDITS_FONT, CREDITS_FONT_SIZE);
	QFont title_font = QFont(CREDITS_FONT, CREDITS_FONT_SIZE);
	title_font.setItalic(true);

	// General info:
	appendCreditText (i18n("Taxipilot Version %1 by Thomas Friedrichsmeier.").arg(VERSION), credits_font, 50, true);

	// Mission info:
	appendCreditText (i18n("This Mission: "), credits_font, 10, false);
	appendCreditText (i18n("'"), credits_font, 0, false);
	appendCreditText (mission_title, title_font, 0, false);
	appendCreditText (i18n("' by ") + mission_author, credits_font, 50, true);

	// Taxi info:
	appendCreditText (i18n("The Taxis: "), credits_font, 10, true);

	Taxipilot::Taxis::ConstIterator ti;
	i = 0;
	for (ti = taxis.begin (); ti != taxis.end (); ti++) {
		Taxi *credits_taxi = new Taxi(ti.data ().filename, 0, 0, true);

		appendCreditText (i18n("'"), credits_font, 0, false);
		appendCreditText (credits_taxi->title, title_font, 0, false);
		appendCreditText (i18n("' ("), credits_font, 0, false);
		appendCreditSprite (credits_taxi, 0, false);
		appendCreditText (i18n(") by ") + credits_taxi->author, credits_font, (++i < taxis.count ()) ? 20 : 50, true);
	}

	// Passenger_info:
	appendCreditText (i18n("The Passengers: "), credits_font, 10, true);

	Taxipilot::Passengers::ConstIterator it;
	i = 0;
	for (it = passengers.begin (); it != passengers.end (); it++) {
		Passenger *credits_passenger = new Passenger(it.data ().filename);
		credits_passenger->initialize (0, 0, 0, true);

		appendCreditText (i18n("'"), credits_font, 0, false);
		appendCreditText (credits_passenger->title, title_font, 0, false);
		appendCreditText (i18n("' ("), credits_font, 0, false);
		appendCreditSprite (credits_passenger, 0, false);
		appendCreditText (i18n(") by ") + credits_passenger->author, credits_font, (++i < passengers.count ()) ? 20 : 50, true);
	}

	// Level-info:
	appendCreditText (i18n("The Levels: "), credits_font, 10, true);

	for (i = 0; i < levels.count (); i++) {
		QString convert;

		if (levels[i].filename == "") continue;
		Screen *credits_level;
		if (levels[i].splash) {
			credits_level = new Level(levels[i].filename);
		} else {
			credits_level = new SplashScreen(levels[i].filename);
		}

		appendCreditText (convert.setNum(i + 1) + ": '", credits_font, 0, false);
		appendCreditText (credits_level->caption, title_font, 0, false);
		appendCreditText (i18n("' by ") + credits_level->author, credits_font, (i < (levels.count () -1)) ? 20 : 50, true);

		delete credits_level;	// not needed
	}

// set off animation
	first_visible = credits.begin ();
	last_visible = credits.begin ();
	tm->startTimer(this, cdp->framePeriod());	// set off animation


}

Credits::~Credits() {
	for (CreditsList::iterator it = credits.begin (); it != credits.end (); ++it) {
		delete (*it)->item;
		delete *it;
	}
	credits.clear ();

	tm->killTimers (this);
}

void Credits::appendCreditSprite(AnimatedSprite * item, int space_after, bool allow_break) {
	CreditItem *dummy = new CreditItem;
	dummy->item = item;
	dummy->space_to_next = space_after;
	dummy->allow_break = allow_break;
	appendCredit(dummy);
}

void Credits::appendCreditText(const QString & text, const QFont &font, int space_after, bool allow_break) {
// TODO: create an AnimatedSprite instead?
	CreditItem *dummy = new CreditItem;
	QCanvasText *text_dummy = new QCanvasText(text, font, canvas);
	text_dummy->setColor(Qt::CREDITS_FONT_COLOR);
	dummy->item = text_dummy;
	dummy->space_to_next = space_after;
	dummy->allow_break = allow_break;
	appendCredit(dummy);
}

void Credits::appendCredit(CreditItem * item) {
//TODO: get rid of this function
	item->item->setAnimated(false);
	item->item->setVelocity(-CREDITS_SPEED / cdp->fps (), 0);
	item->width = item->item->boundingRect().width();
	item->y = view->height() - item->item->boundingRect().height() - 5;
	item->item->move(view->width(), item->y);
	credits.append(item);
}

void Credits::timerTick (int) {
	// start first item if not yet shown
	if (!(*first_visible)->item->visible ()) {
		(*first_visible)->item->show ();
	}

	// trigger new item if necessary
	if (last_visible != credits.fromLast ()) {
		if ((*last_visible)->item->x () < (view->width () - ((*last_visible)->width + (*last_visible)->space_to_next))) {
			++last_visible;
			(*last_visible)->item->show ();
		}
	}

	// hide first_visible if scrolled out
	if ((*first_visible)->item->x () < (- (*first_visible)->width)) {
		(*first_visible)->item->move (view->width (), (*first_visible)->y);
		(*first_visible)->item->hide ();

		if (first_visible != credits.fromLast ()) {
			++first_visible;
		} else {
			first_visible = last_visible = credits.begin ();
		}
	}

	// advance all visible items
	CreditsList::iterator it = first_visible;
	while (it != last_visible) {
		(*it)->item->advance (1);
		++it;
	}
	(*last_visible)->item->advance (1);
}
