/***************************************************************************
                          screen.h  -  description
                             -------------------
    begin                : Tue Dec 10 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCREEN_H
#define SCREEN_H

#include <qcanvas.h>
#include "tpbase.h"

#include <qstring.h>
#include <qmap.h>
#include <qdom.h>

/**Base-class for levels and splash-screens.
  *@author Thomas Friedrichsmeier
  */

class MovingObject;
class State;
class Event;
class HotArea;
class Timer;
class CachablePixmapArray;

class Screen : public QCanvasSprite, public TPBase  {
public: 
/** Constructor. Does not yet read any config (use readScreen). Most of the
	time you'll want to create either a Level or a SplashScreen	directly. */
	Screen();
	~Screen();
/** The id of the most expensive fuel platform (-1 if no fuel available (should never happen!)) */
	int exp_fuel_platform;

/** Number of moving objects */
	unsigned int num_objects;
/** Number of states */
	unsigned int num_states;
/** Number of events */
	unsigned int num_events;
/** Number of timers */
	unsigned int num_timers;
/** Array of pointers to the moving objects */
	MovingObject **objects;
/** Array of pointers to the states */
	State **states;
/** Array of pointers to the events */
	Event **events;
/** Array of pointers to the timers */
	Timer **timers;

/** Caption/Title of this screen/level. Will be displayed in the Taxipilot-Titlebar or maybe elsewhere */
	QString caption;
/** author of this screen/level */
	QString author;

/** Returns a pointer to the state identified with ident. Must never be called while not within read_level () ! */
	State * resolve_state (QString & ident);
/** Returns a pointer to the event identified with ident. Must never be called while not within read_level () ! */
	Event * resolve_event (QString & ident);

/** Clears the state- and event-maps (to be called after all entities were read) */
	void clearMaps ();	
protected:						// Private methods
/** Reads config for this screen (basically everything common to levels
	and splashes) */
	void readScreen(const QDomElement & xdocElement);
/** the background for this level */
	QCanvasPixmap *background;
/** the actual image of the level */
	CachablePixmapArray *image;
/** A map of all the States */
	QMap <QString, State*> state_map;
/** A map of all the Events */
	QMap <QString, Event*> event_map;
};

#endif
