/***************************************************************************
                          tpbase.h  -  description
                             -------------------
    begin                : Son Mai 26 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TPBASE_H
#define TPBASE_H

#include <qstring.h>

class Taxipilot;
class Level;
class Screen;
class Cdp;
class TimeManager;
class SoundWrapper;
class SaveGames;
class QString;
class QCanvas;
class QCanvasView;

/**Keeps a few (static) vars that all or most of the Taxipilot-classes need. Inherited by most classes.
  *@author Thomas Friedrichsmeier
  */

class TPBase {
public: 
	TPBase();
	~TPBase();
	static Taxipilot *tp;
	static Level *level;
	static Screen *screen;
	static Cdp *cdp;
	static TimeManager *tm;
	static SoundWrapper *sw;
	static SaveGames *saves;
	static QCanvas *canvas;
	static QCanvasView *view;
	static QString bad_string_value;			// this one saves creating using constant strings like "#INAVLID#" several times. We just use one.

/** An enum of Debug-Levels. Values need to be assigned in order of reverse significance. */
	enum DebugLevel { Suppress=5, Translation=4, Info=3, Warn=2, Serious=1, Critical=0 };

/** A bitwise OR-able list of properties a sound can have */
	enum SoundProperties { Pitchable=1, Continuous=2, Repetitious=4, Interruptable=8 };

/** A bitwise OR-able list of used to specifiy, which options should be read from config (Read*)
	and what the defaults are (Default*) for a sound*/
	enum SoundConfigOptions { ReadSingleSound=1, DefaultSingleSound=2,
								ReadPitchable=4, DefaultPitchable=8,
								ReadContinuous=16, DefaultContinuous=32,
								ReadRepetitious=64, DefaultRepetitious=128,
								ReadInterruptable=256, DefaultInterruptable=512 };

/** A bitwise OR-able list of used to specifiy, which options should be read from config (Read*)
	and what the defaults are (Default*) for an animation*/
	enum AnimConfigOptions { ReadSound=1,
								ReadAnimContinuous=4, DefaultAnimContinuous=8,
								ReadAnimRepetitions=16,
								ReadAnimOscillating=64, DefaultAnimOscillating=128 };

/** Named list of animation directions (should be obvious) */
	enum AnimDirections { AnimForward=0, AnimReverse=1 };
/** Named list of animation orientations (useful if sprite should look differently head left or right / going fore or back) */
	enum AnimOrientations { AnimLeft=0, AnimRight=1, AnimFore=0, AnimBack=1 };
};

#endif
